from xdjango.contrib.captcha.recaptcha import ReCaptchaGoogle


def validate_captcha(response, remoteip=None):
    api = ReCaptchaGoogle()

    res = api.verify_captcha(response, remoteip)
    return res and ("success" in res.data) and res.data["success"]


from django.db.models import CharField
from django.utils.translation import ugettext_lazy as _

from xdjango.core.validators import validate_phone_number


class PhoneNumberField(CharField):
    default_validators = [validate_phone_number]
    description = _("Phone number")

    def __init__(self, *args, **kwargs):
        # max_length=254 to be compliant with RFCs 3696 and 5321
        kwargs['max_length'] = kwargs.get('max_length', 15)
        super(PhoneNumberField, self).__init__(*args, **kwargs)


from django.forms import TypedMultipleChoiceField, CharField, Field
from django.core.exceptions import ValidationError
from django.utils.encoding import smart_text, force_text
from django.utils.translation import ugettext_lazy as _
from django.forms.widgets import  MultipleHiddenInput, SelectMultiple

from xdjango.core import validators


class TypedMultipleField(TypedMultipleChoiceField):

    def validate(self, value):
        """
        Validates if the input is required.
        """
        if self.required and not value:
            raise ValidationError(self.error_messages['required'], code='required')


class EmailOrPhoneNumberField(CharField):

    default_validators = [validators.validate_email_or_phone_number]

    def clean(self, value):
        value = self.to_python(value).strip()
        return super(EmailOrPhoneNumberField, self).clean(value)


class PhoneNumberField(CharField):

    default_validators = [validators.validate_phone_number]

    def clean(self, value):
        value = self.to_python(value).strip()
        return super(PhoneNumberField, self).clean(value)


class MultiCharField(Field):
    hidden_widget = MultipleHiddenInput
    widget = SelectMultiple
    default_error_messages = {
        'invalid_list': _('Enter a list of values.'),
    }

    def to_python(self, value):
        if not value:
            return []
        elif not isinstance(value, (list, tuple)):
            raise ValidationError(self.error_messages['invalid_list'], code='invalid_list')
        return [smart_text(val) for val in value]

    def validate(self, value):
        """
        Validates that the input is a list or tuple.
        """
        if self.required and not value:
            raise ValidationError(self.error_messages['required'], code='required')
        # Validate that each value in the value list is in self.choices.
        for val in value:
            """
            if not self.valid_value(val):
                raise ValidationError(
                    self.error_messages['invalid_choice'],
                    code='invalid_choice',
                    params={'value': val},
                )
            """
            pass

    def has_changed(self, initial, data):
        if initial is None:
            initial = []
        if data is None:
            data = []
        if len(initial) != len(data):
            return True
        initial_set = set(force_text(value) for value in initial)
        data_set = set(force_text(value) for value in data)
        return data_set != initial_set


from django.conf import settings


class TxtLocalAPI(object):
    API_VERSION = "2.0"
    API_HOST = "api.txtlocal.com"
    API_PORT = 443
    API_HTTPS = True
    API_PROXY = None
    API_PROXY_PORT = None

    API_SEND_URL = "/send"

    USERNAME_PARAM_NAME = "username"
    HASH_PARAM_NAME = "hash"
    API_KEY_PARAM_NAME = "apiKey"
    FORMAT_PARAM_NAME = "format"


    @staticmethod
    def set_proxy(proxy, port):
        '''
        Enable using a Proxy to connect through
        @param $proxy The proxy server
        @param $port The proxy port number
        '''
        TxtLocalAPI.API_PROXY = proxy
        TxtLocalAPI.API_PROXY_PORT = port


    def __init__(self, username=None, hash=None, apiKey=None):
        '''
        Create an instance of the class with the username and key obtained from txtlocal.
        @param $username
        @param $apiKey
        '''
        self.username = username or settings.API_SMS_USERNAME if hasattr(settings, 'API_SMS_USERNAME') else None
        self.hash = hash or settings.API_SMS_HASH if hasattr(settings, 'API_SMS_HASH') else None
        self.apiKey = apiKey or settings.API_SMS_KEY if hasattr(settings, 'API_SMS_KEY') else None
        if not self.hash and not self.apiKey:
            raise ValueError(
                "At least one of API_SMS_HASH/API_SMS_KEY is required."
            )
        self.sender = settings.DEFAULT_SMS_SENDER if hasattr(settings, 'DEFAULT_SMS_SENDER') else None

    def _http(self, method, url, x_headers=None, params=None):
        '''
        HTTP Request to the specified API endpoint
        @param $string $url
        @param $string $x_headers
        @return LatchResponse
        '''
        try:
            # Try to use the new Python3 HTTP library if available
            import http.client as http
            import urllib.parse as urllib
        except:
            # Must be using Python2 so use the appropriate library
            import httplib as http
            import urllib

        if x_headers is not None:
            all_headers = x_headers
        else:
            all_headers = dict()

        if TxtLocalAPI.API_PROXY != None:
            if TxtLocalAPI.API_HTTPS:
                conn = http.HTTPSConnection(TxtLocalAPI.API_PROXY, TxtLocalAPI.API_PROXY_PORT)
                conn.set_tunnel(TxtLocalAPI.API_HOST, TxtLocalAPI.API_PORT)
            else:
                conn = http.HTTPConnection(TxtLocalAPI.API_PROXY, TxtLocalAPI.API_PROXY_PORT)
                url = "http://" + TxtLocalAPI.API_HOST + url
        else:
            if TxtLocalAPI.API_HTTPS:
                conn = http.HTTPSConnection(TxtLocalAPI.API_HOST, TxtLocalAPI.API_PORT)
            else:
                conn = http.HTTPConnection(TxtLocalAPI.API_HOST, TxtLocalAPI.API_PORT)

        try:
            if method == "POST" or method == "PUT":
                all_headers["Content-type"] = "application/x-www-form-urlencoded"

            if params is not None:
                x_params = self.add_auth_params(params)
                parameters = urllib.urlencode(x_params)

                conn.request(method, url, parameters, headers=all_headers)
            else:
                conn.request(method, url, headers=all_headers)

            response = conn.getresponse()

            response_data = response.read().decode('utf8')
            conn.close()
            ret = response_data

        except:
            ret = None

        return ret

    def add_auth_params(self, params):
        all_params = params
        if self.apiKey:
            all_params['apiKey'] = self.apiKey
        else:
            all_params['hash'] = self.hash
        all_params['username'] = self.username

        return all_params


    def sendSms(self, number, message, sender=None, sched=None, test=False, receiptURL=None, custom=None,
                optouts=False, simpleReplyService=False):

        if not sender:
            sender = self.sender

        params = {
            'message': message,
            'numbers': number,
            'sender': sender
        }
        return self._http('POST', self.API_SEND_URL + "/", params=params)


from xdjango.core.sms.backends.txtlocal import TxtLocalAPI


def send_sms(message, sender, recipient_list, fail_silently=False):
    api = TxtLocalAPI()

    api.sendSms(sender=sender, number=recipient_list[0], message=message)


from django.conf.urls import url, include

from rest_framework import routers

from xdjango.contrib.auth import views


router = routers.DefaultRouter()
router.register(r'users', views.UserEmailViewSet)


urlpatterns = [
    url(r'^', include(router.urls)),
    url(r'^sessions/$', views.SessionAPIView.as_view()),
]


import json

from django.conf import settings

from sdklib import SdkBase


class Response():
    def __init__(self, content):
        self.data = json.loads(content)


class ReCaptchaGoogle(SdkBase):

    API_HOST = "www.google.com"
    API_PORT = 443
    API_SCHEME = "https"

    API_SITE_VERIFY_URL = "/recaptcha/api/siteverify"

    def __init__(self, api_key=None, secret_key=None):
        self.apiKey = api_key or settings.RECAPTCHA_API_KEY if hasattr(settings, 'RECAPTCHA_API_KEY') else None
        self.secretKey = secret_key or settings.RECAPTCHA_SECRET_KEY if hasattr(settings, 'RECAPTCHA_SECRET_KEY') else None
        if not self.apiKey or not self.secretKey:
            raise ValueError(
                "RECAPTCHA_API_KEY and RECAPTCHA_SECRET_KEY are required."
            )

    def _http(self, method, url, headers=None, params=None, url_params=None):
        _, data, _ = super(ReCaptchaGoogle, self)._http(method, url, headers, params, url_params)
        encoded_data = data.decode('utf8')
        return Response(encoded_data)

    def verify_captcha(self, captcha_response, remoteip=None):
        """

        :param captcha_response: Required. The user response token provided by the reCAPTCHA to the user and provided to
        your site on.
        :param remoteip: Optional. The user's IP address.
        :return: The response is a JSON object:
                {
                  "success": true|false,
                  "error-codes": [...]   // optional
                }
        """
        params = {"secret": self.secretKey, "response": captcha_response}
        return self._http("POST", self.API_SITE_VERIFY_URL, None, params)


from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model


class AntiBruteForceModelBackend(ModelBackend):
    """
    Authenticates against settings.AUTH_USER_MODEL preventing brute-force attacks.
    UserModel must have 'failed_login_attempts' field
    """
    MAX_FAIL_LOGIN_ATTEMPTS = 50

    def authenticate(self, username=None, password=None, **kwargs):
        UserModel = get_user_model()
        if username is None:
            username = kwargs.get(UserModel.USERNAME_FIELD)
        try:
            user = UserModel._default_manager.get_by_natural_key(username)
            if not (user.failed_login_attempts < self.MAX_FAIL_LOGIN_ATTEMPTS):
                # Reset password
                user.set_password(None)
                user.initialize_failed_attempts_counter()
                # saved
                return
            elif user.check_password(password):
                user.initialize_failed_attempts_counter()
                return user
            else:
                user.increment_failed_attempts_counter()
        except UserModel.DoesNotExist:
            pass
        # Run the default password hasher once to reduce the timing
        # difference between an existing and a non-existing user (#20760).
        UserModel().set_password(password)



from django.conf import settings
from django.http import HttpResponseForbidden, HttpResponseRedirect
from django.utils.http import urlquote
from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import login_required

from xdjango.contrib.auth.decorators import login_suggested


class LoginRequiredMixin(object):
    login_url = None

    @classmethod
    def as_view(cls, **initkwargs):
        view = super(LoginRequiredMixin, cls).as_view(**initkwargs)
        return login_required(view, login_url=cls.login_url)


class LoginSuggestedMixin(object):
    @classmethod
    def as_view(cls, **initkwargs):
        view = super(LoginSuggestedMixin, cls).as_view(**initkwargs)
        return login_suggested(view)


class PermissionRequiredMixin(object):
    """
    A view mixin that verifies if the current logged in user has the specified permission
    by wrapping the ``request.user.has_perm(..)`` method.

    If a `get_object()` method is defined either manually or by including another mixin (for example
    ``SingleObjectMixin``) or ``self.object`` is defiend then the permission will be tested against
    that specific instance.

    .. NOTE: Testing of a permission against a specific object instance requires an authentication backend
             that supports. Please see ``django-guardian`` to add object level permissions to your project.

    The mixin does the following:

        If the user isn't logged in, redirect to settings.LOGIN_URL, passing the current
        absolute path in the query string. Example: /accounts/login/?next=/polls/3/.

        If the `raise_exception` is set to True than rather than redirect to login page
        a `PermisionDenied` (403) is raised.

        If the user is logged in, and passes the permission check than the view is executed
        normally.

    **Example Usage**

        class FitterEditView(PermissionRequiredMixin, UpdateView):
            ...
            ### PermissionRequiredMixin settings
            permission_required = 'fitters.change_fitter'

            ### other view settings
            context_object_name="fitter"
            queryset = Fitter.objects.all()
            form_class = FitterForm
            ...

    **Class Settings**
        `permission_required` - the permission to check of form "<app_label>.<permission codename>"
                                i.e. 'polls.can_vote' for a permission on a model in the polls application.

        `login_url` - the login url of your site
        `redirect_field_name - defaults to "next"
        `raise_exception` - defaults to False - raise PermisionDenied (403) if set to True

    """
    ### default class view settings
    login_url = settings.LOGIN_URL
    raise_exception = False
    permission_required = None
    redirect_field_name=REDIRECT_FIELD_NAME

    def dispatch(self, request, *args, **kwargs):
        # call the parent dispatch first to pre-populate few things before we check for permissions
        original_return_value = super(PermissionRequiredMixin, self).dispatch(request, *args, **kwargs)

        # verify class settings
        if self.permission_required == None or len(self.permission_required.split('.')) != 2:
            raise ImproperlyConfigured("'PermissionRequiredMixin' requires 'permission_required' attribute to be set to '<app_label>.<permission codename>' but is set to '%s' instead" % self.permission_required)

        # verify permission on object instance if needed
        has_permission = False
        if hasattr(self, 'object') and self.object is not None:
            has_permission = request.user.has_perm(self.permission_required, self.object)
        elif hasattr(self, 'get_object') and callable(self.get_object):
            has_permission = request.user.has_perm(self.permission_required, self.get_object())
        else:
            has_permission = request.user.has_perm(self.permission_required)

        # user failed permission
        if not has_permission:
            if self.raise_exception:
                return HttpResponseForbidden()
            else:
                path = urlquote(request.get_full_path())
                tup = self.login_url, self.redirect_field_name, path
                return HttpResponseRedirect("%s?%s=%s" % tup)

        # user passed permission check so just return the result of calling .dispatch()
        return original_return_value



# -*- coding: utf-8 -*-

import re

from django.utils.deconstruct import deconstructible
from django.utils.encoding import force_text
from django.core.validators import validate_ipv46_address, RegexValidator
from django.utils.translation import ugettext_lazy as _
from django.core.validators import validate_email
from django.core.exceptions import ValidationError


@deconstructible
class EmailOrPhoneNumberValidator(object):
    #message = _('Enter a valid email address or phone number.')
    message = _(u'Introduce un correo electrónico o un número de teléfono válidos.')
    code = 'invalid'
    user_regex = re.compile(
        r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*$"  # dot-atom
        r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*"$)',  # quoted-string
        re.IGNORECASE)
    domain_regex = re.compile(
        # max length for domain name labels is 63 characters per RFC 1034
        r'((?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+)(?:[A-Z0-9-]{2,63}(?<!-))$',
        re.IGNORECASE)
    literal_regex = re.compile(
        # literal form, ipv4 or ipv6 address (SMTP 4.1.3)
        r'\[([A-f0-9:\.]+)\]$',
        re.IGNORECASE)
    domain_whitelist = ['localhost']
    phone_number_regex = re.compile(r'^\+?1?\d{9,15}$')

    def __init__(self, message=None, code=None, whitelist=None):
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code
        if whitelist is not None:
            self.domain_whitelist = whitelist

    def __call__(self, value):
        value = force_text(value)

        if self.phone_number_regex.match(value):
            return

        if not value or '@' not in value:
            raise ValidationError(self.message, code=self.code)

        user_part, domain_part = value.rsplit('@', 1)

        if not self.user_regex.match(user_part):
            raise ValidationError(self.message, code=self.code)

        if (domain_part not in self.domain_whitelist and
                not self.validate_domain_part(domain_part)):
            # Try for possible IDN domain-part
            try:
                domain_part = domain_part.encode('idna').decode('ascii')
                if self.validate_domain_part(domain_part):
                    return
            except UnicodeError:
                pass
            raise ValidationError(self.message, code=self.code)

    def validate_domain_part(self, domain_part):
        if self.domain_regex.match(domain_part):
            return True

        literal_match = self.literal_regex.match(domain_part)
        if literal_match:
            ip_address = literal_match.group(1)
            try:
                validate_ipv46_address(ip_address)
                return True
            except ValidationError:
                pass
        return False

    def __eq__(self, other):
        return (
            isinstance(other, EmailOrPhoneNumberValidator) and
            (self.domain_whitelist == other.domain_whitelist) and
            (self.message == other.message) and
            (self.code == other.code)
        )


def is_valid_email(email):
    try:
        validate_email(email)
        return True
    except ValidationError:
        return False


def is_valid_phone_number(phone_number):
    if not phone_number:
        return False
    if EmailOrPhoneNumberValidator.phone_number_regex.match(phone_number):
        return True
    return False

validate_email_or_phone_number = EmailOrPhoneNumberValidator()
#msg = "Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed."
validate_phone_number = RegexValidator(regex=r'^\+?1?\d{9,15}$', message=u"El número de teléfono debe seguir el formato: '+999999999'. Se permiten hasta 15 dígitos.")


from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from django.core.mail import send_mail
from django.core.exceptions import ObjectDoesNotExist
from django.db import models, IntegrityError
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from xdjango.db.models import PhoneNumberField, SafeManager
from xdjango.core.sms import send_sms


class EmailUserManager(BaseUserManager, SafeManager):

    """ Custom manager for EmailUser."""

    def _create_user(self, email, password,
                     is_staff, is_superuser, **extra_fields):
        """ Create and save an EmailUser with the given email and password.
        :param str email: user email
        :param str password: user password
        :param bool is_staff: whether user staff or not
        :param bool is_superuser: whether user admin or not
        :return custom_user.models.EmailUser user: user
        :raise ValueError: email is not set
        """
        now = timezone.now()
        if not email:
            raise ValueError('The given email must be set')
        email = self.normalize_email(email)
        is_active = extra_fields.pop("is_active", True)
        user = self.model(email=email, is_staff=is_staff, is_active=is_active,
                          is_superuser=is_superuser, last_login=now,
                          date_joined=now, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email, password=None, **extra_fields):
        """ Create and save an EmailUser with the given email and password.
        :param str email: user email
        :param str password: user password
        :return custom_user.models.EmailUser user: regular user
        """
        is_staff = extra_fields.pop("is_staff", False)
        return self._create_user(email, password, is_staff, False,
                                 **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        """ Create and save an EmailUser with the given email and password.
        :param str email: user email
        :param str password: user password
        :return custom_user.models.EmailUser user: admin user
        """
        return self._create_user(email, password, True, True,
                                 **extra_fields)


class EmailPhoneUserManager(BaseUserManager):

    """ Custom manager for EmailPhoneUser."""

    # Return existent User if a User already exists?
    silent_create_user = True

    def get_by_natural_key(self, username, silent=False):
        try:
            return self.get(email=username)
        except ObjectDoesNotExist:
            pass
        try:
            return self.get(pk=username)
        except:
            pass
        try:
            return self.get(phone_number=username)
        except:
            if not silent:
                raise

    def delete_user(self, username, silent=True):
        """
        Delete a user, if exists. Otherwise, nothing is done.
        :param username:
        :return:
        """
        user = self.get_by_natural_key(username, silent=silent)
        if user:
            user.delete()

    def _create_user(self, email, phone_number, password,
                     is_staff, is_superuser, **extra_fields):
        """ Create and save an EmailPhoneUser with the given email and password.
        :param str email: user email
        :param str phone_number: user phone_number
        :param str password: user password
        :param bool is_staff: whether user staff or not
        :param bool is_superuser: whether user admin or not
        :return custom_user.models.EmailUser user: user
        :raise ValueError: email or phone_number is not set
        """
        now = timezone.now()
        if email is None and phone_number is None:
            raise ValueError('The given email or phone_number must be set')
        if email is not None:
            email = self.normalize_email(email)
        is_active = extra_fields.pop("is_active", True)
        user = self.model(email=email, is_staff=is_staff, is_active=is_active,
                          is_superuser=is_superuser, last_login=now,
                          date_joined=now, **extra_fields)
        user.set_password(password)
        if phone_number:
            user.phone_number = phone_number
        try:
            user.save(using=self._db)
        except IntegrityError:
            if self.silent_create_user:
                # TODO: return user
                return None
            else:
                raise
        return user

    def create_user(self, email=None, phone_number=None, password=None, **extra_fields):
        """ Create and save an EmailUser with the given email and password.
        :param str email: user email
        :param str phone_number: user phone_number
        :param str password: user password
        :return custom_user.models.EmailPhoneUser user: regular user
        """
        is_staff = extra_fields.pop("is_staff", False)
        return self._create_user(email, phone_number, password, is_staff, False,
                                 **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        """ Create and save an EmailUser with the given email and password.
        :param str email: user email
        :param str password: user password
        :return custom_user.models.EmailPhoneUser user: admin user
        """
        return self._create_user(email, None, password, True, True,
                                 **extra_fields)

    def get_or_create_user(self, email=None, phone_number=None, password=None, **extra_fields):
        """ Get or create and save an EmailUser with the given email and password.
        :param str email: user email
        :param str phone_number: user phone_number
        :param str password: user password
        :return custom_user.models.EmailPhoneUser user: regular user
        """
        try:
            user = self.get(email=email)
        except:
            try:
                user = self.get(phone_number=phone_number)
            except:
                user = None
        if user:
            return user

        return self.create_user(email, phone_number, password, **extra_fields)


class AbstractEmailUser(AbstractBaseUser, PermissionsMixin):

    """ Abstract User with the same behaviour as Django's default User.
    AbstractEmailUser does not have username field. Uses email as the
    USERNAME_FIELD for authentication.
    Use this if you need to extend EmailUser.
    Inherits from both the AbstractBaseUser and PermissionMixin.
    The following attributes are inherited from the superclasses:
        * password
        * last_login
        * is_superuser
    """

    email = models.EmailField(_('email address'), max_length=255,
                              unique=True, db_index=True)
    is_staff = models.BooleanField(
        _('staff status'), default=False, help_text=_(
            'Designates whether the user can log into this admin site.'))
    is_active = models.BooleanField(_('active'), default=True, help_text=_(
        'Designates whether this user should be treated as '
        'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    objects = EmailUserManager()

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def get_full_name(self):
        """ Return the email."""
        return self.email

    def get_short_name(self):
        """ Return the email."""
        return self.email

    def email_user(self, subject, message, from_email=None, **kwargs):
        """ Send an email to this User."""
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def deactivate(self):
        """
        """
        self.is_active = False
        self.save()


class AbstractEmailPhoneUser(AbstractBaseUser, PermissionsMixin):

    """ Abstract User with the same behaviour as Django's default User.
    AbstractEmailUser does not have username field. Uses email as the
    USERNAME_FIELD for authentication.
    Use this if you need to extend EmailUser.
    Inherits from both the AbstractBaseUser and PermissionMixin.
    The following attributes are inherited from the superclasses:
        * password
        * last_login
        * is_superuser
    """
    email = models.EmailField(unique=True, db_index=True, null=True, default=None, blank=True)
    phone_number = PhoneNumberField(unique=True, db_index=True, null=True, default=None, blank=True)
    is_staff = models.BooleanField(
        _('staff status'), default=False, help_text=_(
            'Designates whether the user can log into this admin site.'))
    is_active = models.BooleanField(_('active'), default=True, help_text=_(
        'Designates whether this user should be treated as '
        'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    failed_login_attempts = models.IntegerField(default=0)

    objects = EmailPhoneUserManager()

    USERNAME_FIELD = 'id'
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def __unicode__(self):  # Python 3: def __str__(self):
        return unicode(self.id)

    def email_user(self, subject, message, from_email=None, **kwargs):
        """ Send an email to this User."""
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def sms_user(self, message, sender=None, **kwargs):
        """ Send a sms to this User."""
        send_sms(message, sender, [self.phone_number], **kwargs)

    def has_email(self):
        return self.email

    def has_phone_number(self):
        return self.phone_number

    def increment_failed_attempts_counter(self):
        self.failed_login_attempts += 1
        self.save()

    def initialize_failed_attempts_counter(self):
        self.failed_login_attempts = 0
        self.save()






# -*- coding: utf-8 -*-

from django.forms import (Form, CharField, EmailField,)

from xdjango.forms.fields import EmailOrPhoneNumberField, PhoneNumberField
from xdjango.core.validators import is_valid_email, is_valid_phone_number
from xdjango.contrib.captcha import validate_captcha


FIELD_NAME_MAPPING = {
    'g_recaptcha_response': 'g-recaptcha-response',
}


class ReCaptchaFormValidator(Form):
    g_recaptcha_response = CharField()

    def add_prefix(self, field_name):
        # look up field name; return original if not found
        field_name = FIELD_NAME_MAPPING.get(field_name, field_name)
        return super(ReCaptchaFormValidator, self).add_prefix(field_name)

    def clean(self):
        cleaned_data = super(ReCaptchaFormValidator, self).clean()
        g_recaptcha_response = cleaned_data.get("g_recaptcha_response")

        if not validate_captcha(g_recaptcha_response):
            #msg = "Human verification failed."
            msg = u"Falló la verificación anti robots."
            self.add_error('g_recaptcha_response', msg)


class EmailOrPhoneNumberFormValidator(Form):
    """

    """
    email = EmailField(required=False)
    phone_number = PhoneNumberField(required=False)
    email_or_phone_number = EmailOrPhoneNumberField(required=False)
    user_id = CharField(required=False)

    def clean(self):
        cleaned_data = super(EmailOrPhoneNumberFormValidator, self).clean()
        email = cleaned_data.get("email")
        phone_number = cleaned_data.get("phone_number")
        email_or_phone_number = cleaned_data.get("email_or_phone_number")
        user_id = cleaned_data.get("user_id")

        if not self.errors and not email and not phone_number and not email_or_phone_number and not user_id:
            #msg = "Must put 'at least' one of email or phone number."
            msg = u"Debes facilitar, al menos, tu correo electrónico o tu número de teléfono."
            self.add_error('email', msg)
            self.add_error('phone_number', msg)
            self.add_error('email_or_phone_number', msg)

    def clean_email(self):
        cleaned_data = super(EmailOrPhoneNumberFormValidator, self).clean()
        data = cleaned_data.get("email")
        if data:
            return data
        email_or_phone_number = cleaned_data.get("email_or_phone_number")
        if is_valid_email(email_or_phone_number):
            return email_or_phone_number
        return data

    def clean_phone_number(self):
        cleaned_data = super(EmailOrPhoneNumberFormValidator, self).clean()
        data = cleaned_data.get("phone_number")
        if data:
            return data
        email_or_phone_number = cleaned_data.get("email_or_phone_number")
        if is_valid_phone_number(email_or_phone_number):
            return email_or_phone_number
        return data

    def clean_email_or_phone_number(self):
        cleaned_data = super(EmailOrPhoneNumberFormValidator, self).clean()
        data = cleaned_data.get("email_or_phone_number")
        if data:
            return data
        data = cleaned_data.get("email")
        if data:
            return data
        return cleaned_data.get("phone_number")


from django.db import models


class SafeManager(models.Manager):
    def safe_get(self, **kwargs):
        try:
            return super(SafeManager, self).get(**kwargs)
        except:
            return None

    def get_or_create(self, **kwargs):
        return self.safe_get(**kwargs) or self.create(**kwargs)


from django.contrib.auth import get_user_model

from rest_framework import serializers


class UserSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = get_user_model()
        fields = ('url', 'pk', 'last_login', 'is_superuser', 'email', 'is_staff', 'is_active', 'date_joined')
        read_only_fields = ('last_login', 'is_superuser', 'date_joined', 'pk')
        on_creation_read_fields = ('url', 'email', 'pk')


class SessionSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = get_user_model()
        fields = ('url', 'pk', 'last_login', 'email', 'is_staff', 'is_superuser')


from django.contrib.auth import get_user_model, login, logout, authenticate
from django.core.mail import send_mail

from rest_framework import viewsets, status
from rest_framework.views import APIView
from rest_framework.response import Response

from xdjango.rest_framework.mixins import SecureGetObjectMixin
from xdjango.contrib.auth.serializers import UserSerializer, SessionSerializer
from xdjango.contrib.auth.permissions import IsAuthenticatedOrCreateOnly, IsAdminOrOwnOnly
from xdjango.contrib.auth.validators import LoginFormValidator
from xdjango.contrib.auth.responses import INVALID_CREDENTIALS_ERROR_MSG


class UserEmailViewSet(SecureGetObjectMixin, viewsets.ModelViewSet):
    """
    API endpoint that allows users to be created, viewed or edited.
    """
    queryset = get_user_model().objects.all()
    serializer_class = UserSerializer
    permission_classes = (IsAdminOrOwnOnly,)

    def get_queryset(self):
        queryset = self.queryset
        if not self.request.user.is_superuser and not self.request.user.is_staff:
            queryset = queryset.filter(pk=self.request.user.pk)
        return queryset

    def create(self, request, *args, **kwargs):
        send_email = self.request.query_params.get('send_email', False)
        email = self.request.data.get('email', None)
        existent_user = None

        try:
            existent_user = self.queryset.get(email=email)
            serializer = self.get_serializer(existent_user)
        except get_user_model().DoesNotExist:
            # This code is based to create method of CreateModelMixin
            # 'https://github.com/tomchristie/django-rest-framework/blob/master/rest_framework/mixins.py#L14'
            serializer = self.get_serializer(data=request.data)
            serializer.is_valid(raise_exception=True)
            self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)

        # prepare data to return
        meta = getattr(serializer, 'Meta', None)
        read_fields = getattr(meta, 'on_creation_read_fields', [])
        data = dict()
        for field in serializer.data:
            if field in read_fields:
                data[field] = serializer.data[field]

        if existent_user:
            existent_user.email_user(subject='Someone wants to register', message='Hello World!')
        elif send_email:
            send_mail(subject='Send invitation',
                      message='Hello World!',
                      from_email='some@123.com',
                      recipient_list=[email])
        else:
            send_mail(subject='New created user without sending invitation',
                      message='Hello!',
                      from_email='some@123.com',
                      recipient_list=['ivanprjcts@gmail.com'])

        return Response(data, status=status.HTTP_201_CREATED, headers=headers)


class SessionAPIView(APIView):
    """
    Django Session authentication methods.
    """
    permission_classes = (IsAuthenticatedOrCreateOnly,)

    def get(self, request, format=None):
        user = request.user
        context = {"request": request}
        serializer = SessionSerializer(user, context=context)
        return Response(serializer.data)

    def post(self, request, format=None):
        form = LoginFormValidator(request.data)
        if form.is_valid():
            username = form.cleaned_data['username']
            password = form.cleaned_data['password']

            user = authenticate(username=username, password=password)
            if not user:
                return Response(data=INVALID_CREDENTIALS_ERROR_MSG, status=status.HTTP_401_UNAUTHORIZED)
            else:
                if user.is_active:
                    login(request, user)
                    context = {"request": request}
                    serializer = SessionSerializer(user, context=context)
                    return Response(serializer.data)
                else:
                    return Response(data=INVALID_CREDENTIALS_ERROR_MSG, status=status.HTTP_401_UNAUTHORIZED)
        else:
            return Response(data=form.errors, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, format=None):
        logout(request)
        return Response()


from django.http import Http404


class SecureGetObjectMixin(object):
    """
    Secure get object avoiding information leaks.
    """
    def get_object(self):
        try:
            return super(SecureGetObjectMixin, self).get_object()
        except:
            raise Http404


from setuptools import setup, find_packages


EXCLUDE_FROM_PACKAGES = []

setup(
    name = 'xdjango',
    version = '0.2.1.3',
    description = 'Xtra-django library',
    author='Ivan Martin',
    author_email='ivanmar_91@hotmail.com',
    url='https://bitbucket.org/projectx-team/xdjango',
    zip_safe=False,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
)


from rest_framework import permissions


class IsAdminOrOwnOnly(permissions.BasePermission):
    """
    The request is authenticated as an admin user, or is a request done by own user data.
    """
    def has_permission(self, request, view):
        return (
            request.user.is_authenticated() and
            (request.user.is_superuser or request.user.is_staff or request.method in permissions.SAFE_METHODS)
        )

    def has_object_permission(self, request, view, obj):
        return obj == request.user or request.user.is_staff


class IsAuthenticatedOrCreateOnly(permissions.BasePermission):
    """
    The request is authenticated as an admin user, or is a request done by own user data.
    """
    def has_permission(self, request, view):
        return (
            request.user.is_authenticated() or
            request.method == 'POST'
        )